﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

/*
 * CalcInterface is a class that provides many of the same features
 * found on the TI line of calculators used in gaming.
 * 
 * AUTHOR:      Richard Selneck (aka BlakPilar)
 * LAST EDIT:   09 November 2011
 * LICENSE:
 *      Well, there's really no formal license for this. The only
 *      things you *can't* do are edit the namespace, class name,
 *      or license. Feel free to add or remove methods, or use
 *      the ones I've written in other projects. Heck, if you
 *      really wanted, you could port this over to a WinForms
 *      version. If you do use this in another game/application,
 *      you don't have to give credit to me (but it's greatly
 *      appreciated :D). Also, I'm not responsible for any
 *      undesirable outcomes from using this library.
 */

/*
 * TODO:
 *      Grayscale
 */

namespace Betafy {
    public class CalcInterface {

        /// <summary>
        /// Gets the width of the screen on-calc.
        /// </summary>
        public static int OriginalWidth {
            get { return 96; }
        }
        /// <summary>
        /// Gets the height of the screen on-calc.
        /// </summary>
        public static int OriginalHeight {
            get { return 63; }
        }

        private int scaleFactor = 1;
        /// <summary>
        /// The factor to scale the window size compared to the original calc size.
        /// </summary>
        public int ScaleFactor {
            get {
                return this.scaleFactor;
            }
            set {
                this.scaleFactor = value;
                this.UpdateGraphics();
            }
        }

        /// <summary>
        /// Different states that the interface may be in.
        /// </summary>
        public enum State {
            HomeScreen,
            GameScreen,
        }
        private State gameState;
        /// <summary>
        /// The current state of the interface.
        /// </summary>
        public State GameState {
            get { return this.gameState; }
        }

        /// <summary>
        /// The graphics (which includes the window dimensions) of the game.
        /// </summary>
        public GraphicsDeviceManager GraphicsManager { get; set; }

        /// <summary>
        /// Gets the parent of the current calc-interface class.
        /// </summary>
        public Game Parent { get; private set; }

        /// <summary>
        /// The texture to be used as the pixel.
        /// </summary>
        public Texture2D PixelTexture { get; set; }

        private bool[,] gameScreen;
        /// <summary>
        /// The game screen as a matrix of Boolean values.
        /// </summary>
        public bool[,] GameScreen {
            get { return this.gameScreen; }
            set { this.gameScreen = value; }
        }

        private bool[,] homeScreen;
        /// <summary>
        /// The home screen as a matrix of Boolean values.
        /// </summary>
        public bool[,] HomeScreen {
            get { return this.homeScreen; }
            set { this.homeScreen = value; }
        }

        /// <summary>
        /// Gets the height of the screen matrix.
        /// </summary>
        public int ScreenHeight {
            get { return this.gameScreen.GetLength(0); }
        }

        /// <summary>
        /// Gets the width of the screen matrix.
        /// </summary>
        public int ScreenWidth {
            get { return this.gameScreen.GetLength(1); }
        }

        /// <summary>
        /// Provides a list of pictures for the screen.
        /// </summary>
        public List<Picture> Pictures;

        /// <summary>
        /// Provides a list of sprites for the screen.
        /// </summary>
        public List<Sprite> Sprites;

        /// <summary>
        /// Creates a new instance of the CalcInterface class.
        /// </summary>
        public CalcInterface(Game parent, GraphicsDeviceManager gfxManager,
            Texture2D pixelTexture) {

            this.GraphicsManager = gfxManager;                              //Set our graphics device manager
            this.Parent = parent;                                           //Set our parent 
            this.gameState = State.GameScreen;                              //Set the default game-state
            this.gameScreen = new bool[OriginalHeight, OriginalWidth];      //Give the screen a new, clear value
            this.homeScreen = new bool[OriginalHeight, OriginalWidth];      //Make our game and home screens equal
            this.PixelTexture = pixelTexture;                               //Set our pixel texture
            this.Pictures = new List<Picture>();                            //Give our picture collection a non-null value
            this.Sprites = new List<Sprite>();                              //Give our sprite collection a non-null value

        }

        /// <summary>
        /// Draws to the screen depending on the state of the game.
        /// </summary>
        public void DrawScreen(SpriteBatch spriteBatch) {
            this.DrawScreen(spriteBatch, Color.White);
        }

        /// <summary>
        /// Draws to the screen depending on the state of the game with the specified color.
        /// </summary>
        public void DrawScreen(SpriteBatch spriteBatch, Color color) {

            int width, height;
            bool[,] currScreen;
            if (gameState == State.HomeScreen) {
                height = homeScreen.GetLength(0);
                width = homeScreen.GetLength(1);
                currScreen = homeScreen;
            }
            else { //if (gameState == State.GameScreen)
                height = gameScreen.GetLength(0);
                width = gameScreen.GetLength(1);
                currScreen = gameScreen;
            }

            for (int y = 0; y < height; y++) {                              //For each row
                for (int x = 0; x < width; x++) {                           //For each item in that row
                    if (currScreen[y, x]) {                                 //If pixel (row, column) is on

                        spriteBatch.Draw(this.PixelTexture,
                                new Vector2((float)(x * this.scaleFactor),
                                            (float)(y * this.scaleFactor)),
                                color);                                     //Then draw that pixel
                    }
                }
            }
        }

        /// <summary>
        /// Draws a circle given the specified information.
        /// </summary>
        /// <param name="x">X-coordinate of the center of the circle.</param>
        /// <param name="y">Y-coordinate of the center of the circle.</param>
        /// <param name="radius">Radius of the circle.</param>
        public void DrawCircle(int x, int y, int radius) {
            this.DrawCircle(new Point(x, y), radius);
        }

        /// <summary>
        /// Draws a circle given the specified information.
        /// </summary>
        /// <param name="center">The center of the circle to be drawn.</param>
        /// <param name="radius">The radius of said circle.</param>
        public void DrawCircle(Point center, int radius) {

            this.gameState = State.GameScreen;

            if (radius < 0)                                                 //If the radius is negative
                return;                                                     //Exit

            //Taken from WikiPedia for the RasterCircle
            int f = 1 - radius;
            int ddF_x = 1;
            int ddF_y = -2 * radius;
            int x0 = center.X;
            int y0 = center.Y;
            int x = 0;
            int y = radius;

            this.PxlOn(x0, y0 + radius);
            this.PxlOn(x0, y0 - radius);
            this.PxlOn(x0 + radius, y0);
            this.PxlOn(x0 - radius, y0);

            while (x < y) {

                // ddF_x == 2 * x + 1;
                // ddF_y == -2 * y;
                // f == x*x + y*y - radius*radius + 2*x - y + 1;

                if (f >= 0) {

                    y--;
                    ddF_y += 2;
                    f += ddF_y;

                }

                x++;
                ddF_x += 2;
                f += ddF_x;

                this.PxlOn(x0 + x, y0 + y);
                this.PxlOn(x0 - x, y0 + y);
                this.PxlOn(x0 + x, y0 - y);
                this.PxlOn(x0 - x, y0 - y);
                this.PxlOn(x0 + y, y0 + x);
                this.PxlOn(x0 - y, y0 + x);
                this.PxlOn(x0 + y, y0 - x);
                this.PxlOn(x0 - y, y0 - x);

            } //End while(x < y) loop
        }

        /// <summary>
        /// Draws a line.
        /// </summary>
        /// <param name="x1">X-coordinate of the first point.</param>
        /// <param name="y1">Y-coordinate of the first point.</param>
        /// <param name="x2">X-coordinate of the second point.</param>
        /// <param name="y2">Y-coordinate of the second point.</param>
        public void DrawLine(int x1, int y1, int x2, int y2) {
            this.DrawLine(x1, y1, x2, y2, false);
        }

        /// <summary>
        /// Draws a line.
        /// </summary>
        /// <param name="a">The first point of the line.</param>
        /// <param name="b">The second point of the line.</param>
        public void DrawLine(Point a, Point b) {
            this.DrawLine(a.X, a.Y, b.X, b.Y, false);
        }

        /// <summary>
        /// Draws a line.
        /// </summary>
        /// <param name="x1">X-coordinate of the first point.</param>
        /// <param name="y1">Y-coordinate of the first point.</param>
        /// <param name="x2">X-coordinate of the second point.</param>
        /// <param name="y2">Y-coordinate of the second point.</param>
        /// <param name="erase">Whether or not to erase the line.</param>
        public void DrawLine(int x1, int y1, int x2, int y2, bool erase) {

            this.gameState = State.GameScreen;

            bool isSteep = Math.Abs(y2 - y1) > Math.Abs(x2 - x1);
            if (isSteep && x1 != x2) {
                Swap(ref x1, ref y1);
                Swap(ref x2, ref y2);
            }

            if (x1 > x2) {                                                  //If the line wants to go RTL
                Swap(ref x1, ref x2);                                       //Switch the Xs
                Swap(ref y1, ref y2);                                       //Switch the Ys
            }

            const double precision = 0.5;                                   //Our line precision
            int deltaY = Math.Abs(y2 - y1);                                 //Change in Y
            int deltaX = Math.Abs(x2 - x1);                                 //Change in X
            double slope = (double)deltaY / (double)deltaX;                 //Slope of the line
            double deltaErr = (double)Math.Abs(slope);                      //Error for calculating the points in the line
            double err = 0;                                                 //Used for current error
            int yStep = y1 > y2 ? -1 : 1;                                   //If we should go up or down

            if (x1 == x2) {                                                 //If the line is vertical
                if (y1 > y2)                                                //If the line goes up,
                    Swap(ref y1, ref y2);                                   //Swap the values
                for (int y = y1; y <= y2; y++)                              //For each Y-value
                    //Modify it accordingly
                    if (!erase)
                        PxlOn(x1, y);
                    else
                        PxlOff(x1, y);
            }
            else if (y1 == y2) {                                            //If the line is horizontal
                for (int x = x1; x <= x2; x++)                              //For each X-value
                    //Modify it accordingly
                    if (!erase)
                        PxlOn(x, y1);
                    else
                        PxlOff(x, y1);
            }
            else {                                                          //Otherwise if the line is normal
                int y = y1;                                                 //Start our Y @ y1
                for (int x = x1; x <= x2; x++) {                            //For each X-point in the line
                    if (!erase) {                                           //If we're not going to erase it,
                        if (isSteep)                                        //And it's steep,
                            PxlOn(y, x);                                    //Plot (y, x)
                        else                                                //Otherwise
                            PxlOn(x, y);                                    //Plot (x, y)
                    }
                    else {                                                  //If we're erasing the line,
                        //Do the same as before, but erase instead of plot
                        if (isSteep)
                            PxlOff(y, x);
                        else
                            PxlOff(x, y);
                    }
                    err += deltaErr;                                        //Increment our point-error check
                    if (err >= precision) {                                 //If we're out of our precision,
                        y += yStep;                                         //Increase the Y based on slope
                        err--;                                              //Reset our error
                    }
                }
            }
        }

        /// <summary>
        /// Draws a line.
        /// </summary>
        /// <param name="a">The first point of the line.</param>
        /// <param name="b">The second point of the line.</param>
        /// <param name="erase">Whether or not to erase the line.</param>
        public void DrawLine(Point a, Point b, bool erase) {
            this.DrawLine(a.X, a.Y, b.X, b.Y, erase);
        }

        /// <summary>
        /// Draws a rectangle.
        /// </summary>
        /// <param name="x">X coordinate of the upper left corner.</param>
        /// <param name="y">Y coordinate of the upper left corner.</param>
        /// <param name="width">Width of the rectangle.</param>
        /// <param name="height">Height of the rectangle.</param>
        public void DrawRectangle(int x, int y, int width, int height, bool invert) {
            this.DrawRectangle(new Rectangle(x, y, width, height), invert);
        }

        /// <summary>
        /// Draws a rectangle.
        /// </summary>
        /// <param name="start">The point of the upper left corner.</param>
        /// <param name="width">Width of the rectangle.</param>
        /// <param name="height">Height of the rectangle.</param>
        public void DrawRectangle(Point start, int width, int height, bool invert) {
            this.DrawRectangle(start.X, start.Y, width, height, invert);
        }

        /// <summary>
        /// Draws a rectangle.
        /// </summary>
        /// <param name="rect">The rectangle to draw.</param>
        public void DrawRectangle(Rectangle rect, bool invert) {

            this.gameState = State.GameScreen;

            //Top line
            this.DrawLine(rect.X, rect.Y, rect.X + rect.Width, rect.Y, invert);
            //Right line
            this.DrawLine(rect.X + rect.Width, rect.Y, rect.X + rect.Width, rect.Y + rect.Height, invert);
            //Bottom line
            this.DrawLine(rect.X + rect.Width, rect.Y + rect.Height, rect.X, rect.Y + rect.Height, invert);
            //Left line
            this.DrawLine(rect.X, rect.Y + rect.Height, rect.X, rect.Y, invert);

        }

        /// <summary>
        /// Shades a specified area.
        /// </summary>
        /// <param name="left">Left side of the shaded area.</param>
        /// <param name="right">Right side of the shaded area.</param>
        /// <param name="top">Top of the shaded area.</param>
        /// <param name="bottom">Bottom of the shaded area.</param>
        public void Shade(int left, int right, int top, int bottom) {
            this.gameState = State.GameScreen;
            for (int x = left; x <= right; x++)
                for (int y = top; y <= bottom; y++)
                    PxlOn(x, y);
        }

        /// <summary>
        /// Sets the game state to the home screen.
        /// </summary>
        public void DispHome() {
            this.gameState = State.HomeScreen;
        }

        /// <summary>
        /// Sets the game state to the game screen.
        /// </summary>
        public void DispGame() {
            this.gameState = State.GameScreen;
        }

        /// <summary>
        /// Turns a pixel on at the specified point.
        /// </summary>
        /// <param name="pnt">Point to turn on.</param>
        public void PxlOn(Point pnt) {
            this.PxlOn(pnt.X, pnt.Y);
        }

        /// <summary>
        /// Turns a pixel on at the specified point.
        /// </summary>
        /// <param name="pnt">Point to turn on.</param>
        /// <param name="mark">Similar to the calculator. Value of 1, 2, or 3.</param>
        public void PxlOn(Point pnt, int mark) {
            this.PxlOn(pnt.X, pnt.Y, mark);
        }

        /// <summary>
        /// Turns a pixel on at the specified coordinate.
        /// </summary>
        /// <param name="x">X-coordinate of the pixel to be turned on.</param>
        /// <param name="y">Y-coordinate of the pixel to be turned on.</param>
        public void PxlOn(int x, int y) {
            this.PxlOn(x, y, 1);
        }

        /// <summary>
        /// Turns a pixel on at the specified coordinate.
        /// </summary>
        /// <param name="x">X-coordinate of the pixel to be turned on.</param>
        /// <param name="y">Y-coordinate of the pixel to be turned on.</param>
        /// <param name="mark">Similar to the calculator. Value of 1, 2, or 3.</param>
        public void PxlOn(int x, int y, int mark) {
            if (x < this.ScreenWidth && y < this.ScreenHeight
                && x >= 0 && y >= 0)                                        //If the point is within the screen
            {

                if (mark == 2)                                              //If it's a mark-2 point,
                    this.Mark2(x, y, PxlType.On);                           //Draw a mark-2
                else if (mark == 3)                                         //If it's mark-3,
                    this.Mark3(x, y, PxlType.On);                           //Draw a mark-3
                else {                                                      //Otherwise
                    if (this.gameState == State.GameScreen)                 //If we're on the game screen,
                        this.gameScreen[y, x] = true;                       //Just draw a single point
                    else //if (this.gameState == State.HomeScreen)          //If we're on the home screen,
                        this.homeScreen[y, x] = true;                       //Turn a pixel on on it
                }
            }
        }

        /// <summary>
        /// Turns a pixel off at the specified point.
        /// </summary>
        /// <param name="pnt">Point to turn off.</param>
        public void PxlOff(Point pnt) {
            this.PxlOff(pnt.X, pnt.Y);
        }

        /// <summary>
        /// Turns a pixel off at the specified point.
        /// </summary>
        /// <param name="pnt">Point to turn off.</param>
        /// <param name="mark">Similar to the calculator. Value of 1, 2, or 3.</param>
        public void PxlOff(Point pnt, int mark) {
            this.PxlOff(pnt.X, pnt.Y, mark);
        }

        /// <summary>
        /// Turns a pixel off at the specified coordinate.
        /// </summary>
        /// <param name="x">X-coordinate of the pixel to be turned off.</param>
        /// <param name="y">Y-coordinate of the pixel to be turned off.</param>
        public void PxlOff(int x, int y) {
            this.PxlOff(x, y, 1);
        }

        /// <summary>
        /// Turns a pixel off at the specified coordinate.
        /// </summary>
        /// <param name="x">X-coordinate of the pixel to be turned off.</param>
        /// <param name="y">Y-coordinate of the pixel to be turned off.</param>
        /// <param name="mark">Similar to the calculator. Value of 1, 2, or 3.</param>
        public void PxlOff(int x, int y, int mark) {
            if (x < this.ScreenWidth && y < this.ScreenHeight
                && x >= 0 && y >= 0)                                        //If the point is on the screen
            {

                if (mark == 2)                                              //If it's a mark-2 point,
                    this.Mark2(x, y, PxlType.Off);                          //Draw a mark-2
                else if (mark == 3)                                         //If it's mark-3,
                    this.Mark3(x, y, PxlType.Off);                          //Draw a mark-3
                else {                                                      //Otherwise
                    if (this.gameState == State.GameScreen)                 //If we're on the game screen,
                        this.gameScreen[y, x] = false;                      //Just turn off a single point
                    else //if (this.gameState == State.HomeScreen)          //If we're on the home screen,
                        this.homeScreen[y, x] = false;                      //Turn a pixel off on it
                }
            }
        }

        /// <summary>
        /// Inverts a pixel at the given coordinates.
        /// </summary>
        /// <param name="pnt">The point to invert.</param>
        public void PxlChange(Point pnt) {
            this.PxlChange(pnt.X, pnt.Y);
        }

        /// <summary>
        /// Inverts a pixel at the given coordinates.
        /// </summary>
        /// <param name="pnt">The point to invert.</param>
        /// <param name="mark">Similar to the calculator. Value of 1, 2, or 3.</param>
        public void PxlChange(Point pnt, int mark) {
            this.PxlChange(pnt.X, pnt.Y, mark);
        }

        /// <summary>
        /// Inverts a pixel at the given coordinates.
        /// </summary>
        /// <param name="x">X-coordinate of the pixel.</param>
        /// <param name="y">Y-coordinate of the pixel.</param>
        public void PxlChange(int x, int y) {
            this.PxlChange(x, y, 1);
        }

        /// <summary>
        /// Inverts a pixel at the given coordinates.
        /// </summary>
        /// <param name="x">X-coordinate of the pixel.</param>
        /// <param name="y">Y-coordinate of the pixel.</param>
        /// <param name="mark">Similar to the calculator. Value of 1, 2, or 3.</param>
        public void PxlChange(int x, int y, int mark) {
            if (x < this.ScreenWidth && y < this.ScreenHeight
                && x >= 0 && y >= 0)
            {                                                               //If the point is on the screen

                if (mark == 2)                                              //If it's a mark-2 point,
                    this.Mark2(x, y, PxlType.Invert);                                       //Reverse a mark-2 value
                else if (mark == 3)                                         //If it's a mark-3 point,
                    this.Mark3(x, y, PxlType.Invert);                                       //Reverse a mark-3 value
                else {                                                      //Otherwise
                    if (this.gameState == State.GameScreen)                 //If we're on the game screen,
                        this.gameScreen[y, x] = !this.gameScreen[y, x];     //Invert a pixel on it
                    else //if (this.gameState == State.HomeScreen)          //If we're on the home screen,
                        this.homeScreen[y, x] = !this.homeScreen[y, x];     //Invert a pixel on that one
                }
            }
        }

        /// <summary>
        /// Tests the specified point and returns a value based on it's state.
        /// </summary>
        /// <param name="pnt">The point to test.</param>
        public bool PxlTest(Point pnt) {
            return this.PxlTest(pnt.X, pnt.Y);
        }

        /// <summary>
        /// Tests the specified point and returns a value based on it's state.
        /// </summary>
        /// <param name="x">X-coordinate of the point to test.</param>
        /// <param name="y">Y-coordinate of the point to test.</param>
        public bool PxlTest(int x, int y) {
            if (x >= 0 && x < this.ScreenWidth && y >= 0 && y < this.ScreenHeight)
                return this.gameScreen[y, x];
            else
                return false;
        }

        /// <summary>
        /// Draws a horizontal line.
        /// </summary>
        /// <param name="y">The Y value of the horizontal line.</param>
        public void Horizontal(int y) {
            this.gameState = State.GameScreen;
            for (int x = 0; x < this.ScreenWidth; x++)
                this.gameScreen[y, x] = true;
        }

        /// <summary>
        /// Draws a vertical line.
        /// </summary>
        /// <param name="x">The X value of the line.</param>
        public void Vertical(int x) {
            this.gameState = State.GameScreen;
            for (int y = 0; y < this.ScreenHeight; y++)
                this.gameScreen[y, x] = true;
        }

        /// <summary>
        /// Clears the current screen.
        /// </summary>
        public void ClrScreen() {
            if (this.gameState == State.HomeScreen)
                clear(ref this.homeScreen);
            else //if (this.gameState == State.GameScreen)
                clear(ref this.gameScreen);
        }

        /// <summary>
        /// Returns the integer part of the specified number.
        /// </summary>
        /// <param name="number">Number to retrieve integer part from.</param>
        public long iPart(double number) {

            return (long)number;

        }

        /// <summary>
        /// Returns the fractional part of the specified number.
        /// </summary>
        /// <param name="number">Number to retrieve fractional part from.</param>
        public double fPart(double number) {
            long ipart = iPart(number);
            return number - ipart;
        }

        /// <summary>
        /// Returns one minus the fractional part of the specified number.
        /// </summary>
        /// <param name="number">Number to retrieve remainder part from.</param>
        public double rPart(double number) {
            return (double)1 - fPart(number);
        }

        /// <summary>
        /// Returns the absolute value of the specified number.
        /// </summary>
        /// <param name="number">Number to get absolute value of.</param>
        public double Abs(double number) {
            return Math.Abs(number);
        }

        /// <summary>
        /// Returns a random integer between the specified range.
        /// </summary>
        /// <param name="lower">The lowest return value desired.</param>
        /// <param name="higher">The highest return value desired.</param>
        public int RandInt(int lower, int higher) {
            Random rand = new Random();
            return rand.Next(lower, higher);
        }

        /// <summary>
        /// Returns a random integer between the specified range.
        /// </summary>
        /// <param name="lower">The lowest return value desired.</param>
        /// <param name="higher">The highest return value desired.</param>
        /// <param name="seed">The desired seed for the random number.</param>
        public int RandInt(int lower, int higher, int seed) {
            Random rand = new Random(seed);
            return rand.Next(lower, higher);
        }

        /// <summary>
        /// Returns a number between 0.0 and 1.0.
        /// </summary>
        /// <param name="seed">The desired seed for the random number.</param>
        public double Rand(int seed) {
            Random rand = new Random(seed);
            return rand.NextDouble();
        }

        /// <summary>
        /// Returns a number between 0.0 and 1.0.
        /// </summary>
        public double Rand() {
            Random rand = new Random();
            return rand.NextDouble();
        }

        /// <summary>
        /// Returns the sine of the specified number.
        /// </summary>
        public double Sin(double num) {
            return Math.Sin(num);
        }

        /// <summary>
        /// Returns the cosine of the specified number.
        /// </summary>
        public double Cos(double num) {
            return Math.Cos(num);
        }

        /// <summary>
        /// Returns the tangent of the specified number.
        /// </summary>
        public double Tan(double num) {
            return Math.Tan(num);
        }

        /// <summary>
        /// Returns the specified number rounded down if a decimal part exists.
        /// </summary>
        /// <param name="num">Number to get the floor of.</param>
        public double Floor(double num) {
            return Math.Floor(num);
        }

        /// <summary>
        /// Returns the specified number rounded up if a decimal part exists.
        /// </summary>
        /// <param name="num">Number to find the ceiling of.</param>
        public double Ceiling(double num) {
            return Math.Ceiling(num);
        }

        /// <summary>
        /// Rounds the specified number to the specified number of digits.
        /// </summary>
        /// <param name="num">Number to round.</param>
        /// <param name="decimals">Decimal places to round to.</param>
        public double Round(double num, int decimals) {
            return Math.Round(num, decimals);
        }

        /// <summary>
        /// Returns a substring of a provided string.
        /// </summary>
        /// <param name="str">String to get substring of.</param>
        /// <param name="start">Start index (0 to length of the string minus one).</param>
        /// <param name="len">Length of the substring to return.</param>
        public string Sub(string str, int start, int len) {
            return str.Substring(start, len);
        }

        /// <summary>
        /// Returns the length of the string.
        /// </summary>
        public int Length(string str) {
            return str.Length;
        }

        /// <summary>
        /// Returns the first index of a string to find in the parent string.
        /// </summary>
        /// <param name="orig">String to find the substring in.</param>
        /// <param name="find">String to find in parent string.</param>
        public int InString(string orig, string find) {
            return orig.IndexOf(find);
        }

        /// <summary>
        /// Stores the current screen as a picture.
        /// </summary>
        /// <param name="tag">The identifier for the picture.</param>
        public void StorePic(object tag) {

            bool[,] stoScr = new bool[OriginalHeight, OriginalWidth];
            foreach (Picture pic in this.Pictures)
                if (pic.Tag == tag) {
                    for (int r = 0; r < OriginalHeight; r++)
                        for (int c = 0; c < OriginalWidth; c++)
                            stoScr[r, c] = this.gameScreen[r, c];
                    this.Pictures[this.Pictures.IndexOf(GetPic(tag))].Screen = stoScr;
                    return;
                }

            for (int r = 0; r < OriginalHeight; r++)
                for (int c = 0; c < OriginalWidth; c++)
                    stoScr[r, c] = this.gameScreen[r, c];

            this.Pictures.Add(new Picture(tag, stoScr));
        }

        /// <summary>
        /// Recalls a picture from memory and sets it as the screen information.
        /// </summary>
        /// <param name="tag">The identifier for the picture.</param>
        public void RecallPic(object tag) {

            foreach (Picture pic in this.Pictures)
                if (pic.Tag == tag) {
                    for (int r = 0; r < pic.Screen.GetLength(0); r++)
                        for (int c = 0; c < pic.Screen.GetLength(1); c++)
                            this.gameScreen[r, c] = pic.Screen[r, c];
                    break;
                }
        }

        /// <summary>
        /// Returns a picture with the specified tag (null if such tag cannot be found).
        /// </summary>
        /// <param name="tag">Tag to be found.</param>
        public Picture GetPic(object tag) {
            foreach (Picture pic in this.Pictures)
                if (pic.Tag == tag)
                    return pic;
            return null;
        }

        /// <summary>
        /// Adds a sprite to the sprite collection.
        /// </summary>
        /// <param name="tag">Tag for the new sprite.</param>
        /// <param name="hexValue">Value of the new sprite.</param>
        public void AddSprite(object tag, string hexValue) {
            foreach (Sprite s in this.Sprites)
                if (s.Tag == tag)
                    throw new Exception(String.Format(
                        "A sprite with the tag '{0}' already exists.", tag
                        ));

            this.Sprites.Add(new Sprite(tag, hexValue));
        }

        /// <summary>
        /// Removes a sprite with the designated tag.
        /// </summary>
        /// <param name="tag">Tag of the sprite to remove.</param>
        public void RemoveSprite(object tag) {
            for (int i = 0; i < this.Sprites.Count; i++)
                if (this.Sprites[i].Tag == tag) {
                    this.Sprites.RemoveAt(i);
                    break;
                }
        }

        /// <summary>
        /// Draws the specified sprite to the screen.
        /// </summary>
        /// <param name="sprite">Sprite to draw.</param>
        /// <param name="x">X-coordinate of the sprite's destination.</param>
        /// <param name="y">Y-coordinate of the sprite's destination.</param>
        /// <param name="useTransparency">If true, white pixels will not erase those behind it.</param>
        public void DrawSprite(Sprite sprite, int x, int y, bool useTransparency) {

            int[,] data = sprite.GetData();
            Point pnt = new Point(x, y);

            for (int h = 0; h < data.GetLength(0); h++) {
                for (int w = 0; w < data.GetLength(1); w++) {

                    if (data[h, w] == 1)
                        this.PxlOn(pnt);
                    else
                        if (!useTransparency)
                            this.PxlOff(pnt);

                    pnt.X++;

                }

                pnt.X = x;
                pnt.Y++;
            }
        }

        /// <summary>
        /// Draws the specified sprite to the screen.
        /// </summary>
        /// <param name="sprite">Sprite to draw.</param>
        /// <param name="loc">Location of the sprite's destination.</param>
        /// <param name="useTransparency">If true, white pixels will not erase those behind it.</param>
        public void DrawSprite(Sprite sprite, Point loc, bool useTransparency) {
            this.DrawSprite(sprite, loc.X, loc.Y, useTransparency);
        }

        /// <summary>
        /// Returns the sprite with the specified tag. Null if sprite cannot be found.
        /// </summary>
        /// <param name="tag">Tag of the sprite to find.</param>
        public Sprite GetSprite(object tag) {
            foreach (Sprite s in this.Sprites)
                if (s.Tag == tag)
                    return s;
            return null;
        }

        /// <summary>
        /// Draws text on the game screen. Does not register with the screen data.
        /// </summary>
        /// <param name="text">Text to draw.</param>
        /// <param name="x">X coordinate of the text.</param>
        /// <param name="y">Y coordinate of the text.</param>
        /// <param name="spriteBatch">SpriteBatch to draw the text with.</param>
        public void DrawText(bool isSmall, string text, int x, int y) {
            this.gameState = State.GameScreen;
            int spriteHeight = 6;
            foreach (char ch in text) {

                if (ch == '\n' || x >= ScreenWidth) {
                    x = 0;
                    y += spriteHeight;
                }
                Letter letter = new Letter(ch, !isSmall);
                DrawSprite(letter, x, y, true);
                spriteHeight = letter.Height;

                if (ch != '\n')
                    x += letter.Width;
            }
        }

        /// <summary>
        /// Displays the specified text and switches to the home-screen.
        /// </summary>
        /// <param name="text">Text to display.</param>
        public void Disp(string text) {
            this.Output(0, 0, text);
        }

        /// <summary>
        /// Outputs text to a specified area.
        /// </summary>
        /// <param name="x">X coordinate of the text.</param>
        /// <param name="y">Y coordinate of the text.</param>
        /// <param name="text">The text.</param>
        public void Output(int x, int y, string text) {
            this.gameState = State.HomeScreen;                                  //Change the game state
            int spriteHeight = 8;
            foreach (char ch in text) {

                if (ch == '\n' || x >= ScreenWidth) {             //If it's a new line or we've gone over the screen
                    x = 0;                                                      //Reset the X
                    y += spriteHeight;                                          //Go to the next column
                }
                Letter letter = new Letter(ch, true);                           //Get the letter
                DrawSprite(letter, x, y, false);                                //Draw the letter
                spriteHeight = letter.Height;                                   //Set the height variable

                if (ch != '\n')                                                 //If the character is not a new-line character
                    x += letter.Width;                                          //Increment X by the letter width
            }
        }

        /// <summary>
        /// Returns an array of all pressed keys.
        /// </summary>
        public Keys[] GetKeys() {
            return Keyboard.GetState().GetPressedKeys();
        }

        /// <summary>
        /// Updates the graphics of the game.
        /// </summary>
        public void Update(GraphicsDeviceManager gfxManager, Game parent) {

            if (parent != this.Parent || gfxManager != this.GraphicsManager) {
                this.Parent = parent;
                this.GraphicsManager = gfxManager;

                this.UpdateGraphics();
            }
        }

        /// <summary>
        /// Resets the screens and the picture and sprite lists.
        /// </summary>
        public void Reset() {
            this.gameScreen = new bool[OriginalHeight, OriginalWidth];      //Reset the game screen
            this.homeScreen = new bool[OriginalHeight, OriginalWidth];      //Reset the home screen
            this.Pictures = new List<Picture>();                            //Give our picture collection a non-null value
            this.Sprites = new List<Sprite>();                              //Give our sprite collection a non-null value
        }

        /// <summary>
        /// Draws a mark-2 point. (Box)
        /// </summary>
        /// <param name="x">X of the center.</param>
        /// <param name="y">Y of the center.</param>
        /// <param name="val">Value to make it.</param>
        private void Mark2(int x, int y, PxlType mode) {

            this.gameState = State.GameScreen;

            //The coordinates for the mark-2 point
            int[] xColl = new int[8] {
              x - 1, x, x + 1,
              x - 1,    x + 1,
              x - 1, x, x + 1,
            };
            int[] yColl = new int[8] {
              y - 1, y - 1, y - 1,
              y,            y,
              y + 1, y + 1, y + 1
            };

            for (int i = 0; i < xColl.Length; i++) {
                if (mode == PxlType.Invert)
                    PxlChange(xColl[i], yColl[i]);
                else if (mode == PxlType.Off)
                    PxlOff(xColl[i], yColl[i]);
                else
                    PxlOn(xColl[i], yColl[i]);
            }

        }

        /// <summary>
        /// Draws a mark-3 point. (Cross)
        /// </summary>
        /// <param name="x">X of the center.</param>
        /// <param name="y">Y of the center.</param>
        /// <param name="val">Value to make it.</param>
        private void Mark3(int x, int y, PxlType mode) {

            this.gameState = State.GameScreen;

            int[] xColl = new int[5] {
              x, x - 1, x, x + 1, x
            };
            int[] yColl = new int[5] {
              y - 1, y, y, y, y + 1
            };

            for (int i = 0; i < xColl.Length; i++) {
                if (mode == PxlType.Invert)
                    PxlChange(xColl[i], yColl[i]);
                else if (mode == PxlType.Off)
                    PxlOff(xColl[i], yColl[i]);
                else
                    PxlOn(xColl[i], yColl[i]);
            }
        }

        /// <summary>
        /// Swaps the values of a and b. Mainly used for line algorithm.
        /// </summary>
        private void Swap(ref int a, ref int b) {
            int temp = a;
            a = b;
            b = temp;
        }

        /// <summary>
        /// Clears the referenced matrix.
        /// </summary>
        /// <param name="scr"></param>
        private void clear(ref bool[,] scr) {
            for (int y = 0; y < scr.GetLength(0); y++)
                for (int x = 0; x < scr.GetLength(1); x++)
                    scr[y, x] = false;
        }

        private enum PxlType {
            On,
            Off,
            Invert
        }

        /// <summary>
        /// Modifies the window's width and height.
        /// </summary>
        private void UpdateGraphics() {
            //Change the window dimensions
            this.GraphicsManager.PreferredBackBufferHeight =
                OriginalHeight * this.scaleFactor;              //Window height
            this.GraphicsManager.PreferredBackBufferWidth =
                OriginalWidth * this.scaleFactor;               //Window width
            this.GraphicsManager.ApplyChanges();                //Apply our changes
        }
    }

    /// <summary>
    /// Provides and object for pictures.
    /// </summary>
    public sealed class Picture {

        /// <summary>
        /// Gets the tag (identity) of the picture.
        /// </summary>
        public object Tag { get; private set; }

        bool[,] screen;
        /// <summary>
        /// Gets the information of the picture.
        /// </summary>
        public bool[,] Screen {
            get {
                return this.screen;
            }
            set {
                this.screen = new bool[value.GetLength(0), value.GetLength(1)];
                for (int r = 0; r < value.GetLength(0); r++)
                    for (int c = 0; c < value.GetLength(1); c++)
                        this.screen[r, c] = value[r, c];
            }
        }

        /// <summary>
        /// Creates a new instance of the picture.
        /// </summary>
        /// <param name="tag">Sets the tag for the picture.</param>
        /// <param name="screenInfo">Sets the screen information for the picture.</param>
        public Picture(object tag, bool[,] screenInfo) {
            this.Tag = tag;
            this.Screen = screenInfo;
        }

    }

    /// <summary>
    /// Provides an object for 8x8 sprites.
    /// </summary>
    public class Sprite {

        /// <summary>
        /// Gets the hexadecimal value of the sprite.
        /// </summary>
        public string HexValue { get; set; }

        /// <summary>
        /// Gets the tag (identity) of the sprite.
        /// </summary>
        public object Tag { get; set; }

        /// <summary>
        /// Creates a new sprite with a given hex value.
        /// </summary>
        /// <param name="tag">Tag of the sprite.</param>
        /// <param name="data">Data value of the sprite.</param>
        public Sprite(object tag, string hex) {

            bool isHex = isValidHex(hex);

            if (!isHex)
                throw new Exception("Sprite data is not valid.");

            this.HexValue = hex;
            this.Tag = tag;
        }

        /// <summary>
        /// Returns the sprite as an integer array
        /// </summary>
        /// <returns></returns>
        public int[,] GetData() {

            int[,] data = new int[8, 8];
            string bin = "";
            int y = 0;

            for (int i = 0; i < this.HexValue.Length; i += 2) {             //For each hex number in the data

                bin = this.decToBin(
                    this.hexToDec(this.HexValue.Substring(i, 2)));          //Get the binary value of a section of the hex

                for (int i2 = 0; i2 < bin.Length; i2++)
                    data[y, i2] = "1" == bin.Substring(i2, 1) ? 1 : 0;      //Construct the row @y

                y++;
            }

            return data;                                                    //Return our sprite data
        }

        private int hexToDec(string hex) {
            return int.Parse(hex, System.Globalization.NumberStyles.HexNumber);
        }

        public string decToBin(int dec) {
            string bin = Convert.ToString(dec, 2);
            return zeroes(8 - bin.Length) + bin;
        }

        private string zeroes(int num) {
            string _0 = "0000000000000000";
            if (num >= 0)
                return _0.Substring(0, num);
            else
                return "";
        }

        private bool isValidHex(string data) {
            foreach (char ch in data)
                if (!("0123456789ABCDEF".Contains(ch.ToString())))
                    return false;
            return true;
        }

    }

    /// <summary>
    /// A non-inheritable class for letters.
    /// </summary>
    public sealed class Letter : Sprite {

        /// <summary>
        /// If true, the letter is home-screen-sized. Otherwise, it is graph-screen-sized.
        /// </summary>
        public bool IsLarge { get; private set; }

        /// <summary>
        /// The string equivalent of the letter.
        /// </summary>
        public char ActualText { get; private set; }

        /// <summary>
        /// The actual text of the letter.
        /// </summary>
        public ASCII Value { get; private set; }

        /// <summary>
        /// Width of the letter, counting padding.
        /// </summary>
        public int Width { get; private set; }

        /// <summary>
        /// Height of the letter, counting padding
        /// </summary>
        public int Height { get; private set; }

        /// <summary>
        /// A collection of the letters used on the calculator.
        /// </summary>
        public enum ASCII {
            A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
            a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p, q, r, s, t, u, v, w, x, y, z,
            _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, Theta,
            Underscore, ForwardSlash, BackSlash, Apostrophe, Quote,
            Colon, Semicolon,
            LeftParenthesis, RightParenthesis,
            LeftBracket, RightBracket,
            LeftBrace, RightBrace,
            Period, Comma, Question, Exclamation, Space
        }

        /// <summary>
        /// Creates a new Letter.
        /// </summary>
        /// <param name="text">The ASCII token this will represent.</param>
        public Letter(char text, bool isLarge)
            : base(null, "") {

            this.ActualText = text;
            this.Value = GetASCII(text);
            this.IsLarge = isLarge;
            this.generateData();

        }

        /// <summary>
        /// Generates the data for the current letter.
        /// </summary>
        private void generateData() {
            if (this.HexValue != "")                                    //If the data already exists
                return;                                                 //Exit

            /* Large font taken from http://tibasicdev.wikidot.com/83lgfont
             * Small font taken from http://tibasicdev.wikidot.com/83smfont
             */

            //First, generalize the size. May be changed for some small letters
            if (IsLarge) {
                this.Width = 6;
                this.Height = 8;
            }
            else {
                this.Width = 4;
                this.Height = 6;
            }

            //Then check against each possible value and set hex values
            switch (this.ActualText) {
                #region Numbers
                case '0':
                    this.HexValue = this.IsLarge ? "708898A8C8887000" : "0040A0A0A0400000";
                    break;
                case '1':
                    this.HexValue = this.IsLarge ? "2060202020207000" : "0040C04040E00000";
                    break;
                case '2':
                    this.HexValue = this.IsLarge ? "708808102040F800" : "00C0204080E00000";
                    break;
                case '3':
                    this.HexValue = this.IsLarge ? "F810201008887000" : "00C0204020C00000";
                    break;
                case '4':
                    this.HexValue = this.IsLarge ? "10305090F8101000" : "0080A0E020200000";
                    break;
                case '5':
                    this.HexValue = this.IsLarge ? "F880F00808887000" : "00E080C020C00000";
                    break;
                case '6':
                    this.HexValue = this.IsLarge ? "304080F088887000" : "006080E0A0E00000";
                    break;
                case '7':
                    this.HexValue = this.IsLarge ? "F808102040404000" : "00E0204080800000";
                    break;
                case '8':
                    this.HexValue = this.IsLarge ? "7088887088887000" : "00E0A0E0A0E00000";
                    break;
                case '9':
                    this.HexValue = this.IsLarge ? "7088887808106000" : "00E0A0E020C00000";
                    break;
                #endregion
                #region Capitol letters

                case 'A':
                    this.HexValue = this.IsLarge ? "708888F888888800" : "0040A0E0A0A00000";
                    break;
                case 'B':
                    this.HexValue = this.IsLarge ? "F08888F08888F000" : "00C0A0C0A0C00000";
                    break;
                case 'C':
                    this.HexValue = this.IsLarge ? "7088808080887000" : "0060808080600000";
                    break;
                case 'D':
                    this.HexValue = this.IsLarge ? "F08888888888F000" : "00C0A0A0A0C00000";
                    break;
                case 'E':
                    this.HexValue = this.IsLarge ? "F88080F08080F800" : "00E080C080E00000";
                    break;
                case 'F':
                    this.HexValue = this.IsLarge ? "F88080F080808000" : "00E080C080800000";
                    break;
                case 'G':
                    this.HexValue = this.IsLarge ? "708880B888887800" : "006080A0A0600000";
                    break;
                case 'H':
                    this.HexValue = this.IsLarge ? "888888F888888800" : "00A0A0E0A0A00000";
                    break;
                case 'I':
                    this.HexValue = this.IsLarge ? "7020202020207000" : "00E0404040E00000";
                    break;
                case 'J':
                    this.HexValue = this.IsLarge ? "3810101010906000" : "00202020A0E00000";
                    break;
                case 'K':
                    this.HexValue = this.IsLarge ? "8890A0C0A0908800" : "00A0A0C0A0A00000";
                    break;
                case 'L':
                    this.HexValue = this.IsLarge ? "808080808080F800" : "0080808080E00000";
                    break;
                case 'M':
                    this.HexValue = this.IsLarge ? "88D8A8A888888800" : "00A0E0E0A0A00000";
                    break;
                case 'N':
                    this.HexValue = this.IsLarge ? "8888C8A898888800" : "00C0A0A0A0A00000";
                    break;
                case 'O':
                    this.HexValue = this.IsLarge ? "7088888888887000" : "00E0A0A0A0E00000";
                    break;
                case 'P':
                    this.HexValue = this.IsLarge ? "F08888F080808000" : "00C0A0C080800000";
                    break;
                case 'Q':
                    this.HexValue = this.IsLarge ? "70888888A8906800" : "00E0A0A0E0600000";
                    break;
                case 'R':
                    this.HexValue = this.IsLarge ? "F08888F0A0908800" : "00C0A0C0A0A00000";
                    break;
                case 'S':
                    this.HexValue = this.IsLarge ? "788080700808F000" : "0060804020C00000";
                    break;
                case 'T':
                    this.HexValue = this.IsLarge ? "F820202020202000" : "00E0404040400000";
                    break;
                case 'U':
                    this.HexValue = this.IsLarge ? "8888888888887000" : "00A0A0A0A0E00000";
                    break;
                case 'V':                                                   //Change last A0 to 40 in !IsLarge for original V
                    this.HexValue = this.IsLarge ? "8888888850502000" : "00A0A0A0A0400000";
                    break;
                case 'W':
                    this.HexValue = this.IsLarge ? "888888A8A8A85000" : "00A0A0A0E0A00000";
                    break;
                case 'X':
                    this.HexValue = this.IsLarge ? "8888502050888800" : "00A0A040A0A00000";
                    break;
                case 'Y':
                    this.HexValue = this.IsLarge ? "8888885020202000" : "00A0A04040400000";
                    break;
                case 'Z':
                    this.HexValue = this.IsLarge ? "F80810204080F800" : "00E0204080E00000";
                    break;
                case '\0':                                                  //Theta
                    this.HexValue = this.IsLarge ? "304888F888906000" : "0040A0E0A0400000";
                    break;
                #endregion
                #region Lowercase letters
                case 'a':
                    this.HexValue = this.IsLarge ? "0000700878887800" : "000060A0A0600000";
                    break;
                case 'b':
                    this.HexValue = this.IsLarge ? "8080B0C88888F000" : "0080C0A0A0C00000";
                    break;
                case 'c':
                    this.HexValue = this.IsLarge ? "0000708080887000" : "0000608080600000";
                    break;
                case 'd':
                    this.HexValue = this.IsLarge ? "0808689888887800" : "002060A0A0600000";
                    break;
                case 'e':
                    this.HexValue = this.IsLarge ? "00007088F8807000" : "000040A0C0600000";
                    break;
                case 'f':
                    this.HexValue = this.IsLarge ? "304840E040404000" : "004080C080800000";
                    if (!IsLarge)
                        this.Width = 3;
                    break;
                case 'g':
                    this.HexValue = this.IsLarge ? "0078888878087000" : "0060A06020C00000";
                    break;
                case 'h':
                    this.HexValue = this.IsLarge ? "8080B0C888888800" : "0080C0A0A0A00000";
                    break;
                case 'i':
                    this.HexValue = this.IsLarge ? "2000602020207000" : "0080008080800000";
                    if (!IsLarge)
                        this.Width = 2;
                    break;
                case 'j':
                    this.HexValue = this.IsLarge ? "1000301010906000" : "00200020A0400000";
                    break;
                case 'k':
                    this.HexValue = this.IsLarge ? "4040485060504800" : "008080A0C0A00000";
                    break;
                case 'l':
                    this.HexValue = this.IsLarge ? "6020202020207000" : "00C0404040400000";
                    if (!IsLarge)
                        this.Width = 3;
                    break;
                case 'm':
                    this.HexValue = this.IsLarge ? "0000D0A8A8888800" : "0000D0A8A8880000";
                    this.Width = 6;
                    break;
                case 'n':
                    this.HexValue = this.IsLarge ? "0000B0C888888800" : "0000C0A0A0A00000";
                    break;
                case 'o':
                    this.HexValue = this.IsLarge ? "0000708888887000" : "000040A0A0400000";
                    break;
                case 'p':
                    this.HexValue = this.IsLarge ? "0000F088F0808000" : "0000C0A0C0800000";
                    break;
                case 'q':
                    this.HexValue = this.IsLarge ? "0000689878080800" : "000060A060200000";
                    break;
                case 'r':
                    this.HexValue = this.IsLarge ? "0000B0C880808000" : "0000A0C080800000";
                    break;
                case 's':
                    this.HexValue = this.IsLarge ? "000070807008F000" : "0000C08040C00000";
                    if (!IsLarge)
                        this.Width = 3;
                    break;
                case 't':
                    this.HexValue = this.IsLarge ? "4040E04040483000" : "0080C08080400000";
                    if (!IsLarge)
                        this.Width = 3;
                    break;
                case 'u':
                    this.HexValue = this.IsLarge ? "0000888888986800" : "0000A0A0A0E00000";
                    break;
                case 'v':                                                   //Change last A0 in !IsLarge to 40 for original
                    this.HexValue = this.IsLarge ? "0000888888502000" : "0000A0A0A0400000";
                    break;
                case 'w':
                    this.HexValue = this.IsLarge ? "00008888A8A85000" : "000088A8A8500000";
                    this.Width = 6;
                    break;
                case 'x':
                    this.HexValue = this.IsLarge ? "0000885020508800" : "0000A04040A00000";
                    break;
                case 'y':
                    this.HexValue = this.IsLarge ? "0000888878087000" : "0000A0A040800000";
                    break;
                case 'z':
                    this.HexValue = this.IsLarge ? "0000F8102040F800" : "0000F02040F00000";
                    if (!IsLarge)
                        this.Width = 5;
                    break;

                #endregion
                #region Symbols
                case ';':
                    this.HexValue = this.IsLarge ? "0060600060204000" : "0000400040800000";
                    this.Width = 3;
                    break;
                case ':':
                    this.HexValue = this.IsLarge ? "0060600060600000" : "0000800080000000";
                    this.Width = 2;
                    break;
                case '"':
                    this.HexValue = this.IsLarge ? "5050500000000000" : "00A0A0A000000000";
                    break;
                case '\'':
                    this.HexValue = this.IsLarge ? "2020200000000000" : "0080808000000000";
                    this.Width = 2;
                    break;
                case '/':
                    this.HexValue = this.IsLarge ? "0008102040800000" : "0020204080800000";
                    break;
                case '\\':
                    this.HexValue = this.IsLarge ? "0080402010080000" : "0080804020200000";
                    break;
                case '_':
                    this.HexValue = this.IsLarge ? "000000000000F800" : "0000000000E00000";
                    break;
                case '(':
                    this.HexValue = this.IsLarge ? "1020404040201000" : "0040808080400000";
                    this.Width = 3;
                    break;
                case ')':
                    this.HexValue = this.IsLarge ? "4020101010204000" : "0080404040800000";
                    this.Width = 3;
                    break;
                case '[':
                    this.HexValue = this.IsLarge ? "3020202020203000" : "00C0808080C00000";
                    this.Width = 3;
                    break;
                case ']':
                    this.HexValue = this.IsLarge ? "6020202020206000" : "00C0404040C00000";
                    this.Width = 3;
                    break;
                case '{':
                    this.HexValue = this.IsLarge ? "1820204020201800" : "0060408040600000";
                    break;
                case '}':
                    this.HexValue = this.IsLarge ? "C02020102020C000" : "00C0402040C00000";
                    break;
                case '|':
                    this.HexValue = this.IsLarge ? "2020202020202000" : "0080808080800000";
                    this.Width = 2;
                    break;
                case '?':
                    this.HexValue = this.IsLarge ? "7088081020002000" : "00C0204000400000";
                    break;
                case '!':
                    this.HexValue = this.IsLarge ? "2020202000202000" : "0080808000800000";
                    this.Width = 2;
                    break;
                case ',':
                    this.HexValue = this.IsLarge ? "0000000060204000" : "0000004040800000";
                    this.Width = 3;
                    break;
                case '.':
                    this.HexValue = this.IsLarge ? "0000000060600000" : "0000000000800000";
                    this.Width = 2;
                    break;

                #endregion

                default:                                                //Default is space; ' '
                    this.HexValue = "0000000000000000";
                    if (!IsLarge)
                        this.Width = 2;                                 //Width of space if small
                    break;
            }
        }

        /// <summary>
        /// Returns an ASCII member based on a given character.
        /// </summary>
        /// <param name="chr">Character to get ASCII member from.</param>
        public ASCII GetASCII(char chr) {
            ASCII result;

            switch (chr) {
                case ':':
                    result = ASCII.Colon;
                    break;
                case ';':
                    result = ASCII.Semicolon;
                    break;
                case '"':
                    result = ASCII.Quote;
                    break;
                case '\'':
                    result = ASCII.Apostrophe;
                    break;
                case '\\':
                    result = ASCII.BackSlash;
                    break;
                case '/':
                    result = ASCII.ForwardSlash;
                    break;
                case '_':
                    result = ASCII.Underscore;
                    break;
                case '\0':
                    result = ASCII.Theta;
                    break;
                case '1':
                    result = ASCII._1;
                    break;
                case '2':
                    result = ASCII._2;
                    break;
                case '3':
                    result = ASCII._3;
                    break;
                case '4':
                    result = ASCII._4;
                    break;
                case '5':
                    result = ASCII._5;
                    break;
                case '6':
                    result = ASCII._6;
                    break;
                case '7':
                    result = ASCII._7;
                    break;
                case '8':
                    result = ASCII._8;
                    break;
                case '9':
                    result = ASCII._9;
                    break;
                case '0':
                    result = ASCII._0;
                    break;
                case 'A':
                    result = ASCII.A;
                    break;
                case 'B':
                    result = ASCII.B;
                    break;
                case 'C':
                    result = ASCII.C;
                    break;
                case 'D':
                    result = ASCII.D;
                    break;
                case 'E':
                    result = ASCII.E;
                    break;
                case 'F':
                    result = ASCII.F;
                    break;
                case 'G':
                    result = ASCII.G;
                    break;
                case 'H':
                    result = ASCII.H;
                    break;
                case 'I':
                    result = ASCII.I;
                    break;
                case 'J':
                    result = ASCII.J;
                    break;
                case 'K':
                    result = ASCII.K;
                    break;
                case 'L':
                    result = ASCII.L;
                    break;
                case 'M':
                    result = ASCII.M;
                    break;
                case 'N':
                    result = ASCII.N;
                    break;
                case 'O':
                    result = ASCII.O;
                    break;
                case 'P':
                    result = ASCII.P;
                    break;
                case 'Q':
                    result = ASCII.Q;
                    break;
                case 'R':
                    result = ASCII.R;
                    break;
                case 'S':
                    result = ASCII.S;
                    break;
                case 'T':
                    result = ASCII.T;
                    break;
                case 'U':
                    result = ASCII.U;
                    break;
                case 'V':
                    result = ASCII.V;
                    break;
                case 'W':
                    result = ASCII.W;
                    break;
                case 'X':
                    result = ASCII.X;
                    break;
                case 'Y':
                    result = ASCII.Y;
                    break;
                case 'Z':
                    result = ASCII.Z;
                    break;
                case 'a':
                    result = ASCII.a;
                    break;
                case 'b':
                    result = ASCII.b;
                    break;
                case 'c':
                    result = ASCII.c;
                    break;
                case 'd':
                    result = ASCII.d;
                    break;
                case 'e':
                    result = ASCII.e;
                    break;
                case 'f':
                    result = ASCII.f;
                    break;
                case 'g':
                    result = ASCII.g;
                    break;
                case 'h':
                    result = ASCII.h;
                    break;
                case 'i':
                    result = ASCII.i;
                    break;
                case 'j':
                    result = ASCII.j;
                    break;
                case 'k':
                    result = ASCII.k;
                    break;
                case 'l':
                    result = ASCII.l;
                    break;
                case 'm':
                    result = ASCII.m;
                    break;
                case 'n':
                    result = ASCII.n;
                    break;
                case 'o':
                    result = ASCII.o;
                    break;
                case 'p':
                    result = ASCII.p;
                    break;
                case 'q':
                    result = ASCII.q;
                    break;
                case 'r':
                    result = ASCII.r;
                    break;
                case 's':
                    result = ASCII.s;
                    break;
                case 't':
                    result = ASCII.t;
                    break;
                case 'u':
                    result = ASCII.u;
                    break;
                case 'v':
                    result = ASCII.v;
                    break;
                case 'w':
                    result = ASCII.w;
                    break;
                case 'x':
                    result = ASCII.x;
                    break;
                case 'y':
                    result = ASCII.y;
                    break;
                case 'z':
                    result = ASCII.z;
                    break;
                case '(':
                    result = ASCII.LeftParenthesis;
                    break;
                case ')':
                    result = ASCII.RightParenthesis;
                    break;
                case '[':
                    result = ASCII.LeftBracket;
                    break;
                case ']':
                    result = ASCII.RightBracket;
                    break;
                case '{':
                    result = ASCII.LeftBrace;
                    break;
                case '}':
                    result = ASCII.RightBrace;
                    break;
                case '!':
                    result = ASCII.Exclamation;
                    break;
                case ',':
                    result = ASCII.Comma;
                    break;
                case '.':
                    result = ASCII.Period;
                    break;
                case ' ':
                    result = ASCII.Space;
                    break;
                default:
                    result = ASCII.Question;
                    break;
            }

            return result;
        }
    }
}